// -*- C++ -*-
/*!
 * @file  BetaRnaPort.cpp
 * @brief BetaRna in/outport component
 * $Date$
 *
 * $Id$
 */

#include "BetaRnaPort.h"

// Module specification
// <rtc-template block="module_spec">
static const char* betarnaport_spec[] =
  {
    "implementation_id", "BetaRnaPort",
    "type_name",         "BetaRnaPort",
    "description",       "BetaRna in/outport component",
    "version",           "0.1",
    "vendor",            "",
    "category",          "Generic",
    "activity_type",     "COMMUTATIVE",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

BetaRnaPort::BetaRnaPort(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_DataCountIn("DataCount", m_DataCount),
    m_DataAryIn("DataAry", m_DataAry),
    m_IDIn("ID", m_ID),
    m_ParamDirIn("ParamDir", m_ParamDir),
    m_ResultIDOut("ResultID", m_ResultID),
    m_TrustyOut("Trusty", m_Trusty),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("DataCount", m_DataCountIn);
  registerInPort("DataAry", m_DataAryIn);
  registerInPort("ID", m_IDIn);
  registerInPort("ParamDir", m_ParamDirIn);
  
  // Set OutPort buffer
  registerOutPort("ResultID", m_ResultIDOut);
  registerOutPort("Trusty", m_TrustyOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

BetaRnaPort::~BetaRnaPort()
{
}


/*
RTC::ReturnCode_t BetaRnaPort::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t BetaRnaPort::onActivated(RTC::UniqueId ec_id)
{
  // f[^̓|[gɁA݂݂Ă\̂ŁA
  // f[^̓|[gɑ΂āAxAǂݍݏs
  m_DataCountIn.read();
  m_DataAryIn.read();
  m_IDIn.read();
  m_ParamDirIn.read();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t BetaRnaPort::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t BetaRnaPort::onExecute(RTC::UniqueId ec_id)
{
  static bool bTalker = false;
  static bool bParam = false;
  static int  iUserID = -1;
  static int  iCount = -1;
  int         iRet;
  double      dTrust;

  //------------------------------------------------------------------
  // bIDf[^̓|[gɁAVf[^͂ꂽꍇ
  //------------------------------------------------------------------
  if( m_IDIn.isNew() ) {
    // bIDf[^̓|[gAf[^ǂݍ
    m_IDIn.read();
    
    // bIDf[^Axȏ͂ꂽƂ
    bTalker = true;

    // bIDێ
    iUserID = m_ID.data;
    
    // 샂[h\
    if( iUserID < 0 ) {
      // bID̏ꍇ́A_[hœ
      fprintf( stdout, "Set mode to detect.\n" );
    }
    else {
      // bID̏ꍇ́AwK[hœ
      fprintf( stdout, "Set mode to learn, output = %d.\n", iUserID );
    }
  }

  //------------------------------------------------------------------
  // fBNgpXf[^̓|[gɁAVf[^͂ꂽꍇ
  //------------------------------------------------------------------
  if( m_ParamDirIn.isNew() ) {
    // fBNgpXf[^Af[^ǂݍ
    m_ParamDirIn.read();

    // wKE_IuWFNgɁAfBNgpXݒ肷
    // ( = wKE_IuWFNg̐ݒύX )
    if( m_cBD.SetPath( m_ParamDir.data ) ) {
      
      // wKE_IuWFNgɁAfBNgpXAxȏݒ肳ꂽƂ
      bParam = true;
      
      fprintf( stdout, "Param files <%s>loaded.\n", (char*)m_ParamDir.data );
    }
  }

  //------------------------------------------------------------------
  // ʍڐf[^̓|[gɁAVf[^͂ꂽꍇ
  //------------------------------------------------------------------
  if( m_DataCountIn.isNew() ) { 
    // ʍڐf[^̓|[gAf[^ǂݍ
    m_DataCountIn.read();

    // ʍڐ̃`FbN
    if( iCount != -1 && iCount != m_DataCount.data ) {
      // ʍڐ́ArŕύXĂ͂ȂȂ
      fprintf( stdout, "Input data size cannot be changed.\n" );
      return RTC::RTC_OK;
    }

    // ʍڐێ
    iCount = m_DataCount.data;
  }

  //------------------------------------------------------------------
  // ʍڐf[^̓|[gɁAVf[^͂ꂽꍇ
  //------------------------------------------------------------------
  if( m_DataAryIn.isNew() ) {
    // ʃf[^̓|[gAf[^ǂݍ
    m_DataAryIn.read();

    // ʍڐ`FbN
    if( iCount != (int)m_DataAry.data.length() ) {
      fprintf( stdout, "Input data size is invalid.\n" );
      return RTC::RTC_OK;
    }

    if( iCount != -1 /* ʍڐAxȏ͂ꂽ? */ &&
        bTalker      /* bIDf[^Axȏ͂ꂽ? */ && 
        bParam       /* wKE_IuWFNgɁAfBNgpXAxȏݒ肳ꂽ? */ ) {

      // wKE_IuWFNgɁAʍڐݒ
      if( ! m_cBD.SetChannel( iCount ) )
        return RTC::RTC_OK;

      // wKE_IuWFNgɁAʂnāAwKE_s
      if( ! m_cBD.DriveRna( iUserID, &(m_DataAry.data[0]), &iRet, &dTrust ) )
        return RTC::RTC_OK;

      // wKE_ʂAf[^o̓|[gɐݒ
      m_ResultID.data = iRet;
      // vxAf[^o̓|[gɐݒ
      m_Trusty.data = dTrust;
      
      // wKE_ʂAf[^o̓|[gAo͂
      m_ResultIDOut.write();
      // vxAf[^o̓|[gAo͂
      m_TrustyOut.write();
    }
    else {
        printf( "Beta RNA not ready, set param or talker.\n" );
    }
  }
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t BetaRnaPort::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t BetaRnaPort::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void BetaRnaPortInit(RTC::Manager* manager)
  {
    RTC::Properties profile(betarnaport_spec);
    manager->registerFactory(profile,
                             RTC::Create<BetaRnaPort>,
                             RTC::Delete<BetaRnaPort>);
  }
  
};


