/******************************************************************************
 *      (C)Copyright        AdIn Research, Inc.         1991-1992.            *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rna01def.h
 *
 *  Note       : Beta-RNA model01 Developer's File
 *
 * $Revision: 1.13 $
 * $Author: fujimaki $
 * $Date: 1999/11/05 03:03:48 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(_RNA01DEF_H_)
#define _RNA01DEF_H_

#include "rna00def.h"
#include "rna01usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/********** Sensor Set **********/
typedef struct sset_ {
    RNUM  rnAsens;              /* number of analogue sensor */
    RNUM  rnDsens;              /* number of digital sensor */
    RANA *praAsens;             /* analogue sensors */
    RDIG *prdDsens;             /* digital sensors */
    RID  *priNtbl;              /* array of normalization table ID */
} SSET;

/********** Pattern Set **********/
typedef struct pset_ {
    RMF          *prmApat;      /* analogue pattern (membership function) */
    RDIG         *prdDpat;      /* digital pattern */
    RANA         *praWeight;    /* weights of each pattern */
    RMD          *prgEach;      /* matching degree of each pattern */
    RMD           rgTotal;      /* matching degree of this pattern set */
    RANA          raWeight;     /* weight of this pattern set */
    RID           riSsetOut;    /* output sensor set ID (model01) */
    RID           riCondOut;    /* output condition set ID (model02) */
    GENPTR        pParam;       /* generic pointer */
    struct pset_ *pPrev;        /* pointer to the previous pattern set */
    struct pset_ *pNext;        /* pointer to the next pattern set */
} PSET;

/********** Pattern Table **********/
typedef struct ptbl_ {
    RNUM   rnApat;              /* analogue dimension */
    RNUM   rnDpat;              /* digital dimension */
    RNUM   rnPset;              /* number of pattern set */
    PSET  *pPset;               /* pointer to the first pattern set */
    RID   *priPset;             /* array of pattern set ID */
    RID    riSset;              /* sensor set which mixed output copy to */
    RMTYPE rmtype;              /* matching type */
    RID    riRule;              /* matching rule ID */
    RCYCLE rcycle;              /* integration cycle counter */
    GENPTR pParam;              /* generic pointer */
} PTBL;

/********** Pattern Group **********/
typedef struct pgrp_ {
    RID    riSset;              /* input sensor set ID */
    RNUM   rnPtbl;              /* number of pattern table */
    RNUM   rnFusion;            /* number of pattern sets to fusion */
    RID   *priPtbl;             /* array of pattern table ID */
    RMD   *prgThreshold;        /* integration threshold */
    RMD   *prgDegree;           /* last matching degree */
    RMD   *prgMean;             /* mean of matching degree */
    RMD   *prgInt;              /* integrated matching degree */
    RCYCLE rcycle;              /* cycle counter */
    GENPTR pParam;              /* generic pointer */
} PGRP;

/********** Function Prototype **********/

extern Ferr PROTOTYPE(rcGetPatternSet, (PSET *, RID, PSET **));

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* not _RNA01DEF_H_ */
