/******************************************************************************
 *      Copyright      Advanced Intelligence Research, Inc.   1991-1992       *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rnaL0def.h
 *
 *  Note       : Beta-RNA Learning Module Developer's File
 *
 * $Revision: 1.14 $
 * $Author: fujimaki $
 * $Date: 1999/11/05 03:03:48 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(_RNAL0DEF_H_)
#define _RNAL0DEF_H_

#include "rna01def.h"
#include "rnaL0usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/*--------------------------- Status & Mode (system) ------------------------*/
#define MRL_ACT         (0x01000000L)    /* active/deactive */
#define MRL_HIST        (0x02000000L)    /* use normal hist (<-> sum hist) */
#define MRL_STAT        (0x80000000L)    /* status effective or not */
#define MRL_SYS_MASK    (0xff000000L)    /* system bit mask */
#define MRL_USR_MASK    (0x00ffffffL)    /* user   bit mask */
/*--------------------------- History Buffer Struct -------------------------*/
typedef struct {
    double dSum0;                  /* sum of teaching signal (indication) */
    double dSum0Pre;               /* sum of tsig when last change MF */
    union {
        struct {
            double         dSum1;  /* sum of (data * tsig) */
            double         dSum2;  /* sum of (data * data * tsig) */
        } shist;
        struct {
            unsigned long  ulRes;  /* history buffer resolution */
            float         *pfHist; /* history buffer area */
        } nhist;
    } hdata;
} RLHBUF;
/*--------------------------- Pattern(MF) Parameter -------------------------*/
typedef struct {
    unsigned long  ulStat;      /* status & mode */
    RLHBUF         rlhBuf;      /* History Buffer */
    RLMFP         *prlm;        /* MF learning parameter */
} RLMF;
/*--------------------------- Pattern Set Parameter -------------------------*/
typedef struct {
    unsigned long  ulStat;      /* status & mode */
    TSIG           tsigTotal;   /* total teaching signal (indication) */
    RLMFP         *prlm;        /* MF learning parameter */
} RLPS;
/*--------------------------- Pattern Table Parameter -----------------------*/
typedef struct {
    unsigned long  ulStat;      /* status & mode */
    RLMFP         *prlm;        /* MF learning parameter */
    RLEXT         *prle;        /* extend mode parameter */
    RLWEIGHT      *prlw;        /* weight control parameter */
} RLPT;
/*--------------------------- Pattern Group Parameter -----------------------*/
typedef struct {
    unsigned long  ulStat;      /* status & mode */
    RLMFP         *prlm;        /* MF learning parameter */
    RLEXT         *prle;        /* extend mode parameter */
    RLWEIGHT      *prlw;        /* weight control parameter */
} RLPG;

/*--------------------------- Function Prptptype ----------------------------*/
extern Ferr PROTOTYPE(rlInitPS, (PSET *, RNUM));
extern Ferr PROTOTYPE(rlFreePS, (PSET *, RNUM));
extern Ferr PROTOTYPE(rlCreateMF, (RMF *, RLMF *, unsigned long, RLMFP *));
extern Ferr PROTOTYPE(rlModifyMF,
                      (RMF *, RMF *, RMF *, unsigned long, RLMFP *));
extern Ferr PROTOTYPE(rlCreateOneDataMF, (RMF *, RANA, RLEXT *));
extern Ferr PROTOTYPE(rlCreateMinDataMF, (RMF *, RANA));
extern Ferr PROTOTYPE(rlCompensatePS, (PTBL *));
extern Ferr PROTOTYPE(rlSetHist, (RLMF *, RANA, TSIG, RBOOL *,
                                  unsigned long, RLMFP *));
extern Ferr PROTOTYPE(rlClearHist, (RLMF *, unsigned long));
extern Ferr PROTOTYPE(rlChangeWeightMF, (RMF *, RANA *, RANA, RMD, RBOOL,
                                         TSIG, unsigned long, RLWEIGHT *));
extern Ferr PROTOTYPE(rlGetEntryForSet, (RID, RID, RID, GENPTR *, int *));
extern RANA * PROTOTYPE(rlGetMatchWorkArea, (RNUM));
extern Ferr PROTOTYPE(rlCheckMFParam, (RLMFP *, int));
extern Ferr PROTOTYPE(rlCheckExtParam, (RLEXT *, int));
extern Ferr PROTOTYPE(rlCheckWeightParam, (RLWEIGHT *, int));
extern Ferr PROTOTYPE(rlCheckLearnMode, (unsigned long, int));

extern Ferr PROTOTYPE(rl_GetLearnMode, (RID, RID, RID, unsigned long *, int *));
extern Ferr PROTOTYPE(rl_GetLearnModePG, (RID, unsigned long *));

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* not _RNAL01DEF_H_ */
