/******************************************************************************
 *      Copyright      Advanced Intelligence Research, Inc.   1991-1992       *
 ******************************************************************************
 ******************************************************************************
 *
 *  File name  : rnaS1usr.h
 *
 *  Note       : Beta-RNA Simulator (Type 1)
 *
 * $Revision: 1.22 $
 * $Author: fujimaki $
 * $Date: 1999/11/05 03:03:48 $
 * $Locker:  $
 *
 *****************************************************************************/
#if !defined(RNAS1USR_H)
#define RNAS1USR_H

#include "rnaS0usr.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/**STDEXT_BEGIN**/
/*------------------------ Struct for Beta-RNA Learnig ----------------------*/
typedef struct {
    unsigned long  ulMode;
    unsigned long  ulStat;
    RNUM           rnIn;
    double        *pdMaxIn;
    double        *pdMinIn;
    RNUM           rnGrade;
    RLMFP         *prlm;
    RLEXT         *prle;
    RLWEIGHT      *prlw;
    RID            riSset;
    RID           *priPtbl;
    RID            riPgrp;
    RANA           raSysMax;
    unsigned long  ulMagic;
} RS1DATA;

/*-------------------- Prototype (Prop. External Functions)------------------*/
EXPORT Ferr PUBLIC PROTOTYPE(RS1_Init, (RS1DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_Set,  (RS1DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_Free, (RS1DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_FreeKernelData, (RS1DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_Exec, (RS1DATA *, int, TSIG));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_Prod, (RS1DATA *, int *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_GetLearnInfo, (LTYPE *, RNUM *, RID *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_GetMatchResult, (RS1DATA *, RMD *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_SetInput, (RS1DATA *, double *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_SetDataSelection, (RS1DATA *, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_GetDataSelection, (RS1DATA *, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_SetDataSelectionEach, (RS1DATA *, int, RBOOL));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_GetDataSelectionEach, (RS1DATA *, int, RBOOL *));
EXPORT Ferr PUBLIC PROTOTYPE(rs1SetLearnParam, (RS1DATA *));
/**STDEXT_END**/

EXPORT Ferr PUBLIC PROTOTYPE(RS1_PrintLearnResult, (unsigned long, RS1DATA *, int,
                                             FILE *, int, Ferr));
EXPORT Ferr PUBLIC PROTOTYPE(RS1_PrintInferResult, (unsigned long, RS1DATA *, int, RMD,
                                             FILE *, int, Ferr));

/*-------------------- Prototype (External Functions)------------------------*/
EXPORT Ferr PUBLIC PROTOTYPE(RS1LibInit,
                     (char *, unsigned long, unsigned long, RLMFP *, RLEXT *,
                      RLWEIGHT *, RANA, RS1DATA **, STREAM *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1LibSet,
                     (char *, RID, unsigned long, unsigned long,
                      RLMFP *, RLEXT *, RLWEIGHT *, RS1DATA **, STREAM *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1LibFree, (RS1DATA *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1LibExec, (RS1DATA *, int *, STREAM *));
EXPORT Ferr PUBLIC PROTOTYPE(RS1LibProd, (RS1DATA *, int *, RMD *, STREAM *));

/*-------------------- Macro (External Functions) ---------------------------*/
#define RS1LibSetInfer(sFile, riPgrp, ulMode, pprs1, ppFile) \
       (RS1LibSet((sFile), (riPgrp), (ulMode), 0L, \
                  (RLMFP *)NULL, (RLEXT *)NULL, (RLWEIGHT *)NULL, \
                  (pprs1), (ppFile)))

#if defined(__cplusplus)
}
#endif /* __cplusplus */

#endif /* RNAS1USR_H */
